# -*- coding: utf-8 -*-
import wx
import wx.glcanvas


class Widget(wx.glcanvas.GLCanvas):
    def __init__(self, parent, controller, *args, **kwargs):
        attribList = (wx.glcanvas.WX_GL_RGBA, # RGBA
                wx.glcanvas.WX_GL_DOUBLEBUFFER, # Double Buffered
                wx.glcanvas.WX_GL_DEPTH_SIZE, 32) # 32 bit
        super(Widget, self).__init__(parent, 
                attribList=attribList, *args, **kwargs)
        self.controller=controller
        self.context = wx.glcanvas.GLContext(self)
        # event binding
        self.Bind(wx.EVT_PAINT, self.OnPaint)
        self.Bind(wx.EVT_SIZE, self.OnResize)
        self.Bind(wx.EVT_ERASE_BACKGROUND, self.OnEraseBG)
        self.Bind(wx.EVT_LEFT_DOWN, 
                lambda e: self.SetFocus() or self.controller.onLeftDown(e.x, e.y) and self.Refresh())
        self.Bind(wx.EVT_LEFT_UP, 
                lambda e: self.controller.onLeftUp(e.x, e.y) and self.Refresh())
        self.Bind(wx.EVT_RIGHT_DOWN, 
                lambda e: self.SetFocus() or self.controller.onRightDown(e.x, e.y) and self.Refresh())
        self.Bind(wx.EVT_RIGHT_UP, 
                lambda e: self.controller.onRightUp(e.x, e.y) and self.Refresh())
        self.Bind(wx.EVT_MIDDLE_DOWN, 
                lambda e: self.SetFocus() or self.controller.onMiddleDown(e.x, e.y) and self.Refresh())
        self.Bind(wx.EVT_MIDDLE_UP, 
                lambda e: self.controller.onMiddleUp(e.x, e.y) and self.Refresh())
        self.Bind(wx.EVT_MOTION, 
                lambda e: self.controller.onMotion(e.x, e.y) and self.Refresh())
        self.Bind(wx.EVT_MOUSEWHEEL, 
                lambda e: self.controller.onWheel(-e.WheelRotation) and self.Refresh())
        self.Bind(wx.EVT_KEY_DOWN,  
                lambda e: self.controller.onKeyDown(e.KeyCode) and self.Refresh())

    def OnPaint(self, event):
        dc = wx.PaintDC(self)
        try:
            self.SetCurrent(self.context)
            self.controller.draw()
            self.SwapBuffers()
            event.Skip()
        except wx._core.PyAssertionError as e:
            print('OnPaint', e)

    def OnResize(self, event):
        self.size = self.GetClientSize()
        try:
            self.SetCurrent(self.context)
            self.controller.onResize(self.size.width, self.size.height)
            self.Refresh()
            event.Skip()
        except wx._core.PyAssertionError as e:
            print('OnResize', e)

    def OnEraseBG(self, event):
        pass # Do nothing, to avoid flashing on MSWin


if __name__=="__main__":
    import glglue.sample
    class Frame(wx.Frame):
        def __init__(self, parent, **kwargs):
            super(Frame, self).__init__(parent, **kwargs)
            # setup opengl widget
            self.controller=glglue.sample.SampleController()
            self.glwidget=Widget(self, self.controller)
            # packing
            sizer=wx.BoxSizer(wx.HORIZONTAL)
            self.SetSizer(sizer)
            sizer.Add(self.glwidget, 1, wx.EXPAND)

    app = wx.App(False)
    frame=Frame(None, title='glglue')
    frame.Show()
    app.MainLoop()




import re
import sys
import math
from OpenGL.GL import *

from ..basecontroller import BaseController
from . import targetview
from . import triangle
from . import cube
from . import coord

DELEGATE_PATTERN=re.compile('^on[A-Z]')
VELOCITY=0.1


def to_radian(degree):
    return degree/180.0*math.pi


class Scene(object):
    def __init__(self):
        self.coord=coord.Coord(1.0)
        self.cube=cube.Cube(0.3)
        self.xrot=0
        self.yrot=0

    def onUpdate(self, ms):
        self.yrot+=ms * VELOCITY
        while self.yrot>360.0:
            self.yrot-=360.0
        self.xrot+=ms * VELOCITY * 0.5
        while self.xrot>360.0:
            self.xrot-=360.0

    def draw(self):
        self.coord.draw()
        glRotate(math.sin(to_radian(self.yrot))*180, 0, 1, 0)
        glRotate(math.sin(to_radian(self.xrot))*180, 1, 0, 0)
        self.cube.draw()


class SampleController(object):
    def __init__(self, view=None, root=None):
        view=view or targetview.TargetView()
        self.view=view
        self.root=root or Scene()
        self.isInitialized=False
        self.delegate(view)
        self.delegate(root)

    def delegate(self, to):
        for name in dir(to):  
            if DELEGATE_PATTERN.match(name):
                method = getattr(to, name)  
                setattr(self, name, method)

    def onUpdate(self, ms):
        self.root.onUpdate(ms)

    def onKeyDown(self, key):
        #print("onKeyDown: %x" % key)
        if key==ord('\033'):
            # Escape
            sys.exit()
        if key==ord('q'):
            # q
            sys.exit()
        else:
            print(key)

    def onInitialize(*args):
        pass

    def initilaize(self):
        self.view.onResize()
        glEnable(GL_DEPTH_TEST)
        glClearColor(0.6, 0.6, 0.4, 0.0)
        # 初期化時の呼び出し
        self.onInitialize()

    def draw(self):
        if not self.isInitialized:
            self.initilaize()
            self.isInitialized=True
        # OpenGLバッファのクリア
        glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT)
        # 投影行列のクリア
        glMatrixMode(GL_PROJECTION)
        glLoadIdentity()
        self.view.updateProjection()
        # モデルビュー行列のクリア
        glMatrixMode(GL_MODELVIEW)
        glLoadIdentity()
        self.view.updateView()
        # 描画
        self.root.draw()

        glFlush()




#!/usr/bin/python
# coding: utf-8

import abc


class BaseController(object):
    __metaclass__=abc.ABCMeta
    def __init__(self):
        pass

    @abc.abstractmethod
    def onUpdate(*args):
        """
        updage scene
        """
        pass

    @abc.abstractmethod
    def onLeftDown(self, x, y):
        """
        mouse event
        """
        pass

    @abc.abstractmethod
    def onLeftUp(self, x, y):
        """
        mouse event
        """
        pass

    @abc.abstractmethod
    def onMiddleDown(self, x, y):
        """
        mouse event
        """
        pass

    @abc.abstractmethod
    def onMiddleUp(self, x, y):
        """
        mouse event
        """
        pass

    @abc.abstractmethod
    def onRightDown(self, x, y):
        """
        mouse event
        """
        pass

    @abc.abstractmethod
    def onRightUp(self, x, y):
        """
        mouse event
        """
        pass

    @abc.abstractmethod
    def onMotion(self, x, y):
        """
        mouse event
        """
        pass

    @abc.abstractmethod
    def onResize(self, w, h):
        """
        gui event
        """
        pass

    @abc.abstractmethod
    def onWheel(self, d):
        """
        mouse event
        """
        pass

    @abc.abstractmethod
    def onKeyDown(*args):
        """
        keyboard event
        """
        pass

    @abc.abstractmethod
    def draw(self):
        """
        draw scene
        """
        pass





import pygame
from pygame.locals import *
import glglue.sample


if __name__=="__main__":   
    pygame.init()
    size=(640, 480)
    pygame.display.gl_set_attribute(pygame.GL_STENCIL_SIZE, 2)
    screen = pygame.display.set_mode(size, 
            HWSURFACE | OPENGL | DOUBLEBUF)

    controller=glglue.sample.SampleController()
    controller.onResize(*size)

    clock = pygame.time.Clock()    
    is_running=True
    while is_running:
        #pressed = pygame.key.get_pressed()

        # event handling
        for event in pygame.event.get():
            if event.type == QUIT:
                is_running=False
            if event.type == KEYDOWN:
                if event.key == K_ESCAPE:
                    is_running=False
                else:
                    controller.onKeyDown(event.key)
            
        # update
        d = clock.tick()
        if d>0:
            controller.onUpdate(d)
            controller.draw()
            pygame.display.flip()




# coding: utf-8
from OpenGL.GL import *


class Triangle(object):
    def __init__(self, size):
        self.size=size

    def draw(self):
        # 三角形描画開始
        glBegin(GL_TRIANGLES)
        # 左下
        glVertex(-self.size, -self.size)
        # 右下
        glVertex(self.size, -self.size)
        # 上
        glVertex(0, self.size)
        # 三角形描画終了
        glEnd()




# coding: utf-8
from OpenGL.GL import *


class Cube(object):
    def __init__(self, s):
        self.vertices=[
                -s, -s, -s,
                 s, -s, -s,
                 s,  s, -s,
                -s,  s, -s,
                -s, -s,  s,
                 s, -s,  s,
                 s,  s,  s,
                -s,  s,  s,
                ]
        self.colors=[
                0, 0, 0,
                1, 0, 0,
                0, 1, 0,
                0, 0, 1,
                0, 1, 1,
                1, 0, 1,
                1, 1, 1,
                1, 1, 0,
                ]
        self.indices=[
                0, 1, 2, 2, 3, 0,
                0, 4, 5, 5, 1, 0,
                1, 5, 6, 6, 2, 1,
                2, 6, 7, 7, 3, 2,
                3, 7, 4, 4, 0, 3,
                4, 7, 6, 6, 5, 4,
                ]

    def draw(self):
        glEnableClientState(GL_VERTEX_ARRAY);
        glVertexPointer(3, GL_FLOAT, 0, self.vertices);
        glEnableClientState(GL_COLOR_ARRAY);
        glColorPointer(3, GL_FLOAT, 0, self.colors);
        glDrawElements(GL_TRIANGLES, len(self.indices), GL_UNSIGNED_INT, self.indices);
        glDisableClientState(GL_COLOR_ARRAY);
        glDisableClientState(GL_VERTEX_ARRAY);




#!/usr/bin/python
# coding: utf-8

from OpenGL.GL import *
from OpenGL.GLU import *


class TargetView(object):
    def __init__(self, distance=None):
        # screen size
        self.w=1
        self.h=1
        # mouse status
        self.x=0
        self.y=0
        self.isLeftDown=False
        self.isMiddleDown=False
        self.isRightDown=False
        # target view params
        self.distance=distance or 5
        self.head=0
        self.pitch=0
        self.shift_x=0
        self.shift_y=0
        self.shift_factor=0.8
        # perspective projection params
        self.fovy=30
        self.aspect=1
        self.near=0.1
        self.far=100

    def updateProjection(self):
        gluPerspective(self.fovy, self.aspect, self.near, self.far)

    def updateView(self):
        glTranslate(self.shift_x, self.shift_y, -self.distance)
        glRotate(self.pitch, 1, 0, 0)
        glRotate(self.head, 0, 1, 0)

    def onResize(self, w=None, h=None):
        print("resize: %d, %d" % (self.w, self.h))
        self.w=w or self.w
        self.h=h or self.h
        glViewport(0, 0, self.w, self.h)
        self.aspect=self.w / float(self.h)

    def onLeftDown(self, x, y):
        self.isLeftDown=True
        self.x=x
        self.y=y

    def onLeftUp(self, x, y):
        self.isLeftDown=False

    def onMiddleDown(self, x, y):
        self.isMiddleDown=True
        self.x=x
        self.y=y

    def onMiddleUp(self, x, y):
        self.isMiddleDown=False

    def onRightDown(self, x, y):
        self.isRightDown=True
        self.x=x
        self.y=y

    def onRightUp(self, x, y):
        self.isRightDown=False

    def onMotion(self, x, y):
        isUpdated=False
        dx=x-self.x
        dy=y-self.y
        if self.isLeftDown:
            if dy>0:
                self.distance*=1.1
            else:
                self.distance*=0.9
            isUpdated=True
        if self.isMiddleDown:
            self.shift_x+=dx / float(self.w) * self.shift_factor * self.distance
            self.shift_y-=dy / float(self.h) * self.shift_factor * self.distance
            isUpdated=True
        if self.isRightDown:
            self.head+=dx
            self.pitch+=dy
            isUpdated=True
        self.x=x
        self.y=y
        return isUpdated

    def onWheel(self, d):
        if d>0:
            self.distance*=1.1
            return True
        elif d<0:
            self.distance*=0.9
            return True

    def printMatrix(self, m):
        print(m[0][0], m[0][1], m[0][2], m[0][3])
        print(m[1][0], m[1][1], m[1][2], m[1][3])
        print(m[2][0], m[2][1], m[2][2], m[2][3])
        print(m[3][0], m[3][1], m[3][2], m[3][3])




#!/usr/bin/env python
# coding: utf-8

from setuptools import setup, find_packages
import sys

setup(
        name='glglue',
        version='0.3.1',
        description='The glue code which mediates between OpenGL and some GUI',
        long_description=open('README.rst').read(),
        classifiers=[
            'Programming Language :: Python :: 2',
            'Programming Language :: Python :: 3',
            'License :: OSI Approved :: zlib/libpng License',
            'Topic :: Multimedia :: Graphics :: 3D Modeling',
            ],
        keywords=['opengl'],
        author='ousttrue',
        author_email='ousttrue@gmail.com',
        license='zlib',
        packages=find_packages(),
        test_suite='nose.collector',
        tests_require=['Nose'],
        )

